# by DCL manual
#english
=begin
==NumRu::AdvancedDCL::Bar
==About this module
   This is the module for drawing bar graphs. An abstract is same as ((<AdvancedDCL::Plot>)).
===Parameters
Name  Type     DCL package
:frtype(Numeric)<UUPACK::UUSFRT>
   Specifies the line type of a frame. Specified attributes are used in Bar.frame,Box.frame methods.
:frindex(Numeric)<UUPACK::UUSFRI>
   Specifies the line index of a frame. Specified attributes are used in Bar.frame,Box.frame methods.
:barwidth(Numeric)<UUPACK::UUSBRS>
   Specifies the horizontal width of a bar graph. 
:lineindex(Numeric)<UUPACK::UUSLNI>
   Specifies the line index of a polyline
:linetype(Numeric)<UUPACK::UUSLNT>
   Specifies the line type of a polyline
:variablerange(Array[2, Numeric])<UUPACK::UUSIDV>
   Specifies the defined range of the independent variable. 
:uref(Float)<UUPACK>
   When the dependent variable is RUNDEF, the value of the internal variable uref (initial value = 0) is considered as specified.
:itpat(Array[2, Numeric])<UUPACK::UUSARP>
   Specifies the tone pattern for filling the internal area. The number set here is used by Gap.tone, Bar.tone, Box.tone. 
:iundef (Fixnum) <SYSLIB>
   Integer-type value indicating 'not specified'. (Initial value: -999)
:rundef (Float) <SYSLIB>
   Floating-point-number-type value indicating 'not specified'. (Initial value: -999.0)
:lmiss (true or false) <SYSLIB>
   Specifies whether missing-value handling will be performed. Performed when true, and not performed when false (Initial value: false).
:imiss (Fixnum) <SYSLIB>
   Specifies the integer-type missing value for missing-value handling. (Initial value: 999)
:rmiss (Float) <SYSLIB>
   Specifies the floating-point-number-type missing value for missing-value handling. (Initial value: 999.0)

Array[2, Float] represents a float Array object whose length is 2.

In addition, These parameters can be used as methods. For example, it is possible to refer the parameter "frtype" with Bar.frtype method. Moreover, it is possible to set the parameter "frtype" with Bar.frtype= method.

=end
#japanese
=begin
==NumRu::AdvancedDCL::Bar
=={W[ɂ
BarW[͖_Ot`߂̃W[łBTv((<AdvancedDCL::Plot>))Ɠ.
===p^
p^ p^̌^ ΉDCLpbP[W
:frtype(Numeric)<UUPACK::UUSFRT>
   g̃C^Cvw肷. ݒ肷鑮 Bar.frame,Box.frame Ŏg. 
:frindex(Numeric)<UUPACK::UUSFRI>
   g̃CCfNXw肷. ݒ肷鑮 Bar.frame,Box.frame Ŏg.
:barwidth(Numeric)<UUPACK::UUSBRS>
   _Ot̉Ђw肷.
:lineindex(Numeric)<UUPACK::UUSLNI>
   ܂̃CCfNXw肷. 
:linetype(Numeric)<UUPACK::UUSLNT>
   ܂̃C^Cvw肷B
:variablerange(Array[2, Numeric])<UUPACK::UUSIDV>
   Ɨϐ̒`w肷. 
:uref(Float)<UUPACK>
   Bar.(x,y)frm,Bar.(x,y)tone,Bar.(x,y)linëX1, Y1 ܂ X2, Y2  RUNDEF w肷, p^ uref ̒lw肳ꂽ̂Ƃ݂Ȃ. uref ̏l 0 ł. 
:itpat(Array[2, Numeric])<UUPACK::UUSARP>
   ̈hg[p^[w肷. Őݒ肷ԍ Gap.tone,Bar.tone,Box.tone Ŏg. 
:iundef (Fixnum) <SYSLIB>
   sł邱Ƃ^̒l(l-999)
:rundef (float) <SYSLIB>
   sł邱Ƃ^̒l(l-999.0)
:lmiss (true or false) <SYSLIB>
   lȂ ǂw肷. true Ȃ炨Ȃ, false Ȃ炨ȂȂ (l false).
:imiss (Fixnum) <SYSLIB>
   lȂƂ̐^̌lw肷 (l999).
:rmiss (Float) <SYSLIB>
   lȂƂ̎^̌lw肷 (l999.0).

ȂAArray[2, Float]ȂǂƂ\L͒gFloat^̔zł钷QArraył邱ƂӖĂB

܂Ãp^̓\bhƂėp邱ƂłBȂ킿ABar.frtypeŃp^frtype̎QƂABar.frtype=Ńp^frtype̕ύXłB

=end

module NumRu
  module AdvancedDCL
    module Bar
      def param_init
	@param = Parameter.new(Common.param)
	@param.init("frtype", Numeric, "uusfrt", "uuqfrt", 1, 1)
	@param.init("frindex", Numeric, "uusfri", "uuqfri", 1, 1)
	@param.init("barwidth", Numeric, "uusbrs", "uuqbrs", 1, 1)
	@param.init("lineindex", Numeric, "lineindex", "uuqlni", 3, 1)
	@param.init("linetype", Numeric, "linetype", "uuqlnt", 3, 1)
	
	@param.init("variablerange", [2, Numeric], "variablerange", "uuqidv", 3, 1)
	@param.init("uref", Float, "uurset", "uurget", 0, 0)
	@param.init("itpat", [2, Numeric], "uusarp", "uuqarp", 2, 1)
	
	@param.init("lmiss", TrueClass, "gllset", "gllget", 0, 0)
	@param.init("imiss", Fixnum, "gliset", "gliget", 0, 0)
	@param.init("rmiss", Float, "glrset", "glrget", 0, 0)
	@param.init("rundef", Float, "glrset", "glrget", 0, 0)
	@param.init("iundef", Fixnum, "gliset", "gliget", 0, 0)

	@temp_param = @param.dup
	@backup = @param.dup
	@orig_param = @param.dup
	@set = Hash.new
      end
    
      private :param_init

#------------------- method for parameters----------------------

      include Parameter_appli

# by kawanabe
#english
=begin
---Bar.set(key, val)
   Sets parameters. If val is nil, the value of the parameter will return default value in DCL.

   Arguments
   * key: parameter name(String)
   * val: new value. About type, refer each parameter.

   Example
      Bar.set("frindex", 3)      # Sets the line index of a frame 3.

=end
#japanese
=begin
---Bar.set(key, val)
    p^̐ݒsBvalnilw肷ƒlDCLɂ鏉lɖ߂B

    
    * key:p^(String)
    * val:ݒ肷lB^͊ep^QƂ̂ƁB
    gp
        Bar.set("frindex", 3)      # g̃CCfNX3ɐݒ肷

=end

#english
=begin
---Bar.get(key)
    Refers the value of the parameter.

    Arguments
    * key: parameter name(String)

    Example
        Bar.get("frindex")      # Refers the line index of a frame.

=end
#japanese
=begin
---Bar.get(key)
    p^QƂB

    
    * key:p^(String)
    gp
        Bar.get("frindex")      #g̃CCfNXQƂ

=end

# by kawanabe
#english
=begin
---Bar.reset(key)
    Sets the value of the parameter default value.

    Arguments
    * key: parameter name(String)

    Example
        Bar.reset("frindex")      # Sets the line index of a frame a default value.

=end
#japanese
=begin
---Bar.reset(key, val)
    p^DCLɂ鏉lɖ߂B

    
    * key:p^(String)
    * val:ݒ肷lB^͊ep^QƂ̂ƁB

    gp
        Bar.reset("frindex")      # g̃CCfNXlɖ߂

=end

#english
=begin
---Bar.help_params
    Refers available parameters in Bar module.

    Arguments
    None

    Notes
    (1)This method return name of the parameter, present value, type of the parameter.

=end
#japanese
=begin
---Bar.help_params
    BarŐݒ\ȃp^QƂB

    
    Ȃ

    l
    (1)̃\bh͐ݒ\ȃp^ɊւāAp^A݂̒lAp^̌^ԂB

=end

#english
=begin
---Bar.sync
    Synchronizes the value of a parameter in DCL with the present value of the parameter in AdvancedDCL.

    Arguments
     None

    Notes
     In AdvancedDCL, parameters which set with "set" method do not synchronize with DCL and a method to synchronize parameters is called in drawing methods. This method is used to synchronize parameters with these of DCL directly.

=end 
#japanese
=begin
---Bar.sync
    ݁Aݒ肳Ăp^DCLɔfB

    
     Ȃ

    l
     AdvancedDCLɂĂset\bĥ݂ł̓p^DCLɔfꂸA
p^sync郁\bhŌĂłB̃\bh̓[Up
^zsyncł邽߂̃\bhłB

=end

#------------ draw method ----------------------

      def yfrm(x1, x2, y, temp=nil)
	rundef = Common.rundef
	if(x1.is_a?(NArray))
	  x1max = x1.max.to_f
	  x1min = x1.min.to_f
	elsif(x1.is_a?(Array))
	  x1max = x1[-1].to_f
	  x1min = x1[0].to_f
	end
	if(x2.is_a?(NArray))
	  x2max = x2.max.to_f
	  x2min = x2.min.to_f
	elsif(x2.is_a?(Array))
	  x2max = x2[-1].to_f
	  x2min = x2[0].to_f
	end
	if(x1==rundef || x2==rundef)
	  x = rundef
	else
	  x=Array.new(2)
	  if(x1min<x2min)
	    x[0]=x1min
	  else
	    x[0]=x2min
	  end
	  if(x1max>x2max)
	    x[1]=x1max
	  else
	    x[1]=x2max
	  end
	end
	Basics.checkundef(x, y)
	
	@param.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.uhbrf(x1.length, x1, x2, y)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual (modified a little)
#english
=begin
---Bar.yfrm(x1, x2, y, temp=nil)
    Draws bar graphs as Y is an independent variable.

    Arguments
    * x1, x2 (Array or NArray) : The coordinate values for both ends of a bar.
    * y (Array or NArray) : Independent variables.
    * temp: temporary parameters (Hash)

    Notes
    (1)UHBRF is called in this method.
    (2)When RUNDEF is specified as Y, it is 
  considered that the points dividing the range defined by the parameter 'variablerange' into N-1 equal sections have been specified. If the defined range is not specified, then the whole window is considered to be the defined range.
    (3)When RUNDEF is specified as X1 or X2, it is considered that the value of the internal variable 'uref' has been specified. The initial value of 'uref' is 0.
    (4)The attributes of the frame drawn by Bar.xfrm, Bar.yfrm are specified by the parameters 'frtype' and 'frindex'. The width of the bar is specified by the parameter 'barwidth'.

=end 
#japanese
=begin
---Bar.yfrm(x1, x2, y, temp=nil)
    YƗϐƂ_Ot̘g`.

    
    * x1, x2 (Array or NArray) : _̗[̍Wl.
    * y (Array or NArray) : Ɨϐ.
    * temp: e|p^(Hash)

    l
    (1)̃\bhUHBRFĂłB
    (2)y RUNDEF w肷, p^variablerangeŎw肵`(x1.length)_ w肳ꂽ̂Ƃ݂Ȃ. `悪w肳ĂȂ, EChEŜ`Ƃ. 
    (3)x1܂ x2 RUNDEF w肷, p^ uref ̒lw肳ꂽ̂Ƃ݂Ȃ. uref ̏l 0 ł. 
    (4)g̑̓p^frtype, frindexŎw肷. _̕barwidthŎw肷.

=end 
    
      def xfrm(x, y1, y2, temp=nil)
	rundef = Common.rundef
	if(y1.is_a?(NArray))
	  y1max = y1.max.to_f
	  y1min = y1.min.to_f
	elsif(y1.is_a?(Array))
	  y1max = y1[-1].to_f
	  y1min = y1[0].to_f
	end
	if(y2.is_a?(NArray))
	  y2max = y2.max.to_f
	  y2min = y2.min.to_f
	elsif(y2.is_a?(Array))
	  y2max = y2[-1].to_f
	  y2min = y2[0].to_f
	end
	if(y1==rundef || y2==rundef)
	  y = rundef
	else
	  y=Array.new(2)
	  if(y1min<y2min)
	    y[0]=y1min
	  else
	    y[0]=y2min
	  end
	  if(y1max>y2max)
	    y[1]=y1max
	  else
	    y[1]=y2max
	  end
	end
	Basics.checkundef(x, y)
      
	@param.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf

	DCL.uvbrf(y1.length, x, y1, y2)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual (modified a little)
#english
=begin
---Bar.xfrm(x, y1, y2, temp=nil)
    Draws bar graphs as X is an independent variable.

    Arguments
    * y1, y2 (Array or NArray) : The coordinate values for both ends of a bar.
    * x (Array or NArray) : Independent variables.
    * temp: temporary parameters (Hash)

    Notes
    (1)UVBRF is called in this method.
    (2)When RUNDEF is specified as X, it is 
  considered that the points dividing the range defined by the parameter 'variablerange' into N-1 equal sections have been specified. If the defined range is not specified, then the whole window is considered to be the defined range.
    (3)When RUNDEF is specified as Y1 or Y2, it is considered that the value of the internal variable 'uref' has been specified. The initial value of 'uref' is 0.
    (4)The attributes of the frame drawn by Bar.xfrm, Bar.yfrm are specified by the parameters 'frtype' and 'frindex'. The width of the bar is specified by the parameter 'barwidth'.

=end 
#japanese
=begin
---Bar.xfrm(x, y1, y2, temp=nil)
    XƗϐƂ_Ot̘g`.

    
    * y1, y2 (Array or NArray) : _̗[̍Wl.
    * x (Array or NArray) : Ɨϐ.
    * temp: e|p^(Hash)

    l
    (1)̃\bhUVBRFĂłB
    (2)x RUNDEF w肷, p^variablerangeŎw肵`(y1.length)_ w肳ꂽ̂Ƃ݂Ȃ. `悪w肳ĂȂ, EChEŜ`Ƃ. 
    (3)y1܂ y2 RUNDEF w肷, p^ uref ̒lw肳ꂽ̂Ƃ݂Ȃ. uref ̏l 0 ł. 
    (4)g̑̓p^frtype, frindexŎw肷. _̕barwidthŎw肷.

=end 
    
      def ytone(x1, x2, y, temp=nil)
	rundef = Common.rundef
	if(x1.is_a?(NArray))
	  x1max = x1.max.to_f
	  x1min = x1.min.to_f
	elsif(x1.is_a?(Array))
	  x1max = x1[-1].to_f
	  x1min = x1[0].to_f
	end
	if(x2.is_a?(NArray))
	  x2max = x2.max.to_f
	  x2min = x2.min.to_f
	elsif(x2.is_a?(Array))
	  x2max = x2[-1].to_f
	  x2min = x2[0].to_f
	end
	if(x1==rundef || x2==rundef)
	  x = rundef
	else
	  x=Array.new(2)
	  if(x1min<x2min)
	    x[0]=x1min
	  else
	    x[0]=x2min
	  end
	  if(x1max>x2max)
	    x[1]=x1max
	  else
	    x[1]=x2max
	  end
	end
	Basics.checkundef(x, y)
	
	@param.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.uhbra(x1.length, x1, x2, y)

	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end
      
# by DCL manual (modified a little)
#english
=begin
---Bar.ytone(x1, x2, y, temp=nil)
    Fills in the interior of a bar graph.

    Arguments
    * x1, x2 (Array or NArray) : The coordinate values for both ends of a bar.
    * y (Array or NArray) : Independent variables.
    * temp: temporary parameters(Hash)

    Notes
    (1)UVBRA is called in this method.
    (2)When RUNDEF is specified as Y, it is considered that the points dividing the range defined by the parameter 'variablerange' into N-1 equal sections have been specified. If the defined range is not specified, then the whole window is considered to be the defined range.
    (3)When RUNDEF is specified as X1 or X2, it is considered that the value of the internal variable 'uref' has been specified. The initial value of 'uref' is 0.
    (4)The tone pattern used for fill by Bar.xtone, Bar.ytone is specified by the parameter 'tpat'. The width of a bar is specified by the parameter 'barwidth'.

=end 
#japanese
=begin
---Bar.ytone(x1, x2, y, temp=nil)
    _Ot̓̈h.

    
    * x1, x2 (Array or NArray) : _̗[̍Wl.
    * y (Array or NArray) : Ɨϐ.
    * temp: e|p^(Hash)

    l
    (1)̃\bhUVBRAĂłB
    (2)y RUNDEF w肷, p^variablerangeŎw肵`(x1.length)_ w肳ꂽ̂Ƃ݂Ȃ. `悪w肳ĂȂ, EChEŜ`Ƃ. 
    (3)x1܂ x2 RUNDEF w肷, p^ uref ̒lw肳ꂽ̂Ƃ݂Ȃ. uref ̏l 0 ł. 
    (4)g[p^[̓p^tpatw肷. _̕barwidthŎw肷.

=end 

      def xtone(x, y1, y2, temp=nil)
	rundef = Common.rundef
	if(y1.is_a?(NArray))
	  y1max = y1.max.to_f
	  y1min = y1.min.to_f
	elsif(y1.is_a?(Array))
	  y1max = y1[-1].to_f
	  y1min = y1[0].to_f
	end
	if(y2.is_a?(NArray))
	  y2max = y2.max.to_f
	  y2min = y2.min.to_f
	elsif(y2.is_a?(Array))
	  y2max = y2[-1].to_f
	  y2min = y2[0].to_f
	end
	if(y1==rundef || y2==rundef)
	  y = rundef
	else
	  y=Array.new(2)
	  if(y1min<y2min)
	    y[0]=y1min
	  else
	    y[0]=y2min
	  end
	  if(y1max>y2max)
	    y[1]=y1max
	  else
	    y[1]=y2max
	  end
	end
	Basics.checkundef(x, y)
	@param.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end

	Fig.settrf
	
	DCL.uvbra(y1.length, x, y1, y2)
	
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end
      
# by DCL manual (modified a little)
#english
=begin
---Bar.xtone(x, y1, y2, temp=nil)
    Fills in the interior of a bar graph.

    Arguments
    * y1, y2 (Array or NArray) : The coordinate values for both ends of a bar.
    * x (Array or NArray) : Independent variables.
    * temp: temporary parameters(Hash)

    Notes
    (1)UHBRA is called in this method.
    (2)When RUNDEF is specified as X, it is considered that the points dividing the range defined by the parameter 'variablerange' into N-1 equal sections have been specified. If the defined range is not specified, then the whole window is considered to be the defined range.
    (3)When RUNDEF is specified as Y1 or Y2, it is considered that the value of the internal variable 'uref' has been specified. The initial value of 'uref' is 0.
    (4)The tone pattern used for fill by Bar.xtone, Bar.ytone is specified by the parameter 'tpat'. The width of a bar is specified by the parameter 'barwidth'.

=end 
#japanese
=begin
---Bar.xtone(x, y1, y2, temp=nil)
    _Ot̓̈h.

    
    * y1, y2 (Array or NArray) : _̗[̍Wl.
    * x (Array or NArray) : Ɨϐ.
    * temp: e|p^(Hash)

    l
    (1)̃\bhUHBRAĂłB
    (2)x RUNDEF w肷, p^variablerangeŎw肵`(y1.length)_ w肳ꂽ̂Ƃ݂Ȃ. `悪w肳ĂȂ, EChEŜ`Ƃ. 
    (3)y1܂ y2 RUNDEF w肷, p^ uref ̒lw肳ꂽ̂Ƃ݂Ȃ. uref ̏l 0 ł. 
    (4)g[p^[̓p^tpatw肷. _̕barwidthŎw肷.

=end 

      def yline(xd, y, temp=nil)
	Basics.checkundef(xd, y)

	@param.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.uhbrl(xd.length, xd, y)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual (modified a little)
#english
=begin
---Bar.yline(xd, y, temp=nil)
    Connects bar graphs with lines. 

    Arguments
    * xd (Array or NArray) : The values for the ends of a bar.
    * y (Array or NArray) : Independent variables.
    * temp: temporary parameters (Hash)

    Notes
    (1)UVBRL is called in this method.
    (2)When RUNDEF is specified as Y, it is considered that the points dividing the range defined by the parameter 'variablerange' into N-1 equal sections have been specified. If the defined range is not specified, then the whole window is considered to be the defined range. 
    (3)When RUNDEF is specified as X1 or X2, it is considered that the value of the internal variable 'uref' has been specified. The initial value of 'uref' is 0.
    (4)The attributes of the line drawn are specified by the parameters 'frype' and 'frindex' respectively. The width of a bar is specified by the parameter 'barwidth'.

=end 
#japanese
=begin
---Bar.yline(xd, y, temp=nil)
    _OtŌ.  

    
    * xd (Array or NArray) : _̒[̒l.
    * y (Array or NArray) : Ɨϐ.
    * temp: e|p^(Hash)

    l
    (1)̃\bhUVBRLĂłB
    (2)y RUNDEF w肷, p^variablerangeŎw肵`(x1.length)_ w肳ꂽ̂Ƃ݂Ȃ. `悪w肳ĂȂ, EChEŜ`Ƃ. 
    (3)xd RUNDEF w肷, p^ uref ̒lw肳ꂽ̂Ƃ݂Ȃ. uref ̏l 0 ł. 
    (4)̑̓p^linetype, lineindexŎw肷. _̕ barwidth Ŏw肷. 

=end 

      def xline(x, yd, temp=nil)
	Basics.checkundef(x, yd)
	
	@param.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.uvbrl(yd.length, x, yd)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual (modified a little)
#english
=begin
---Bar.xline(x, yd, temp=nil)
    Connects bar graphs with lines. 

    Arguments
    * yd (Array or NArray) : The values for the ends of a bar.
    * x (Array or NArray) : Independent variables.
    * temp: temporary parameters (Hash)

    Notes
    (1)UHBRL is called in this method.
    (2)When RUNDEF is specified as X, it is considered that the points dividing the range defined by the parameter 'variablerange' into N-1 equal sections have been specified. If the defined range is not specified, then the whole window is considered to be the defined range. 
    (3)When RUNDEF is specified as Y1 or Y2, it is considered that the value of the internal variable 'uref' has been specified. The initial value of 'uref' is 0.
    (4)The attributes of the line drawn are specified by the parameters 'frype' and 'frindex' respectively. The width of a bar is specified by the parameter 'barwidth'.

=end 
#japanese
=begin
---Bar.xline(x, yd, temp=nil)
    _OtŌ.

    
    * yd (Array or NArray) : _̒[̒l.
    * x (Array or NArray) : Ɨϐ.
    * temp: e|p^(Hash)

    l
    (1)̃\bhUHBRLĂłB
    (2)x RUNDEF w肷, p^variablerangeŎw肵`(y1.length)_ w肳ꂽ̂Ƃ݂Ȃ. `悪w肳ĂȂ, EChEŜ`Ƃ. 
    (3)yd RUNDEF w肷, p^ uref ̒lw肳ꂽ̂Ƃ݂Ȃ. uref ̏l 0 ł. 
    (4)̑̓p^linetype, lineindexŎw肷. _̕barwidthŎw肷.

=end 

      module_function :param_init
      module_function :help_params
      module_function :set
      module_function :get
      module_function :sync
      module_function :reset
      module_function :xfrm
      module_function :yfrm
      module_function :xtone
      module_function :ytone
      module_function :xline
      module_function :yline
      
      param_init
    
#----------------------------------------------------------

      def @param.lineindex(index)
	DCL.uuslni(index)
	DCL.sgspli(index)
	nil
      end
      
      def @temp_param.lineindex(index)
	DCL.uuslni(index)
	DCL.sgspli(index)
	nil
      end
      
      def @backup.lineindex(index)
	DCL.uuslni(index)
	DCL.sgspli(index)
	nil
      end
      
      def @param.linetype(type)
	DCL.uuslnt(type)
	DCL.sgsplt(type)
	nil
      end
      
      def @temp_param.linetype(type)
	DCL.uuslnt(type)
	DCL.sgsplt(type)
	nil
      end
      
      def @backup.linetype(type)
	DCL.uuslnt(type)
	DCL.sgsplt(type)
	nil
      end
      
      def @param.variablerange(range)
	if(range[0]<range[1])
	  DCL.uusidv(range[0], range[1])
	else
	  DCL.uusidv(range[1], range[0])
	end
	nil
      end
      
      def @temp_param.variablerange(range)
	if(range[0]<range[1])
	  DCL.uusidv(range[0], range[1])
	else
	  DCL.uusidv(range[1], range[0])
	end
	nil
      end
      
      def @backup.variablerange(range)
	if(range[0]<range[1])
	  DCL.uusidv(range[0], range[1])
	else
	  DCL.uusidv(range[1], range[0])
	end
	nil
      end

      @param.keys.each{|i|
	eval "def #{i}; @param['#{i}']; end"
	eval "def #{i}=(val); set('#{i}', val); end"
      }
      
      @param.keys.each{|i|
	eval "module_function :#{i}"
	eval "module_function :#{i}="
      }
    end
  end
end
